/* unix.h - Unix domain socket communication */
 
/* Written 1995-1998 by Werner Almesberger, EPFL-LRC/ICA */
 

#ifndef _UNIX_H
#define _UNIX_H

/*--------------------------- Unix domain sockets ---------------------------*/

int un_create(const char *path,mode_t mode);

/*
 * Creates a Unix domain DGRAM socket, binds it to the specified path, and
 * returns the socket descriptor. Returns a negative value on error.
 */

int un_attach(const char *path);

/*
 * Creates a Unix domain DGRAM socket and connects it to the specified path.
 * The local side is bound to an ephemeral address. Returns the socket
 * descriptor on success, a negative value otherwise.
 */

int un_recv_connect(int s,void *buf,int size);

/*
 * Performs a recv(s,buf,size,0) and connects the socket to the sender's
 * address. Returns a negative value on error.
 */

int un_reply(int s,void *buf,int _size,
  int (*handler)(void *buf,int len,void *user),void *user);

/*
 * Receives a message from the socket into the buffer provided by the caller,
 * invokes handler for processing and optionally sends back a reply. If the
 * handler returns a negative value or zero, no reply is sent. If the handler
 * returns a positive value, this is interpreted as the length of the reply to
 * send. The data is taken from the buffer. If any system call fails, un_reply
 * returns a negative value. Otherwise, is returns whatever was returned by the
 * handler function.
 */

#endif
