/* inatmarp.c - InATMARP tool */

/* Written 01-1998 by Francis.Dupont@inria.fr, INRIA */


#include <sys/types.h>
#include <sys/socket.h>
#include <sys/sockio.h>
#include <sys/uio.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/signal.h>
#include <net/if.h>
#include <netinet/in.h>
#include <netinet/in_var.h>
#include <netnatm/natm.h>
#include <arpa/inet.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <stdlib.h>
#include <errno.h>

#include "inatmarp.h"

u_char llcsnap[8] = { 0xaa, 0xaa, 0x03, 0x00, 0x00, 0x00, 0x08, 0x06 };

void dodump(struct aar *aa, u_char *buf, int len)
{
	register int off;
	struct timeval tv;
	struct tm *t;
	time_t clk;
	struct in_addr ia;

	(void) gettimeofday(&tv, NULL);
	clk = (time_t)tv.tv_sec;
	t = localtime(&clk);
	printf("%02d:%02d:%02d.%06u ",
	       t->tm_hour,
	       t->tm_min,
	       t->tm_sec,
	       (u_int)tv.tv_usec);
	switch (aa->op) {
	case AAREQUEST:		/* code = 8 */
	  printf("request: ");
	  break;
	case AAREPLY:		/* code = 9 */
	  printf("reply: ");
	  break;
	}
	off = aa->shtl & 0x3f;
	off += aa->sstl & 0x3f;
	if ((aa->shtl != 0) || (aa->sstl != 0))
		printf("[%x+%x] ", aa->shtl, aa->sstl);
	else
		printf("[] ");
	bcopy(aa->spa + off, &ia, 4);
	off += aa->thtl & 0x3f;
	off += aa->tstl & 0x3f;
	if ((aa->thtl != 0) || (aa->tstl != 0))
		printf("[%x+%x] ", aa->thtl, aa->tstl);
	else
		printf("[] ");
	printf("%s ", inet_ntoa(ia));
	bcopy(aa->tpa + off, &ia, 4);
	if ((aa->op == AAREQUEST) && (ia.s_addr != INADDR_ANY))
		printf("?");
	printf("%s", inet_ntoa(ia));
	off += 4;
	if (aa->tpa + off != buf + len)
		printf(" ...");
	printf("\n");
}

void doreply(int sd, struct aar *aa, struct sockaddr_in *sock_in, 
	     struct in_addr *ia, u_char *buf, int dump, int len)
{
	register int cc, off;

	if ((aa->thtl & 0x3f) || (aa->tstl & 0x3f))
		return;

	aa->hrd = htons(aa->hrd);
	aa->pro = htons(aa->pro);
	aa->thtl = aa->shtl;
	aa->sstl = aa->tstl;
	aa->op = htons(AAREPLY);
	aa->shtl = 0;
	aa->tstl = 0;
	off = aa->thtl & 0x3f;
	off += aa->tstl & 0x3f;
	bcopy(aa->spa + off, ia, 4);
	bcopy(aa->spa, aa->tpa, off);  /* if off = 0 nothing is copied */
	bcopy(ia, aa->tpa + off, 4);
	bcopy(&sock_in->sin_addr, aa->spa, 4);

	cc = sizeof(llcsnap) + sizeof(*aa) + off;
	if (len < cc)
		return;
	len = cc;

	cc = write(sd, buf, len);
	if (cc < 0) {
		perror("write(reply)");
		exit(1);
	}
	if (cc != len) {
		fprintf(stderr,
			"truncated write(reply) (%d != %d)\n",
			cc, len);
		exit(1);
	}
	if (dump) {
		printf("send reply\n");
		dodump(aa, buf, len);
	}
}

void dorequest(int sd, struct aar *aa, struct sockaddr_in *sock_in, 
	       u_char *buf, int dump)
{
	register int len, cc;
	struct in_addr ia;

	len = sizeof(llcsnap) + sizeof(struct aar);

	bzero(buf, sizeof(buf));
	ia.s_addr = INADDR_ANY;
	bcopy(llcsnap, buf, sizeof(llcsnap));
	aa = (struct aar *)(buf + sizeof(llcsnap));
	aa->hrd = htons(0x0013);
	aa->pro = htons(0x0800);
	aa->shtl = 0;
	aa->sstl = 0;
	aa->op = htons(AAREQUEST);
	aa->spln = 4;
	aa->thtl = 0;
	aa->tstl = 0;
	aa->tpln = 4;
	bcopy(&sock_in->sin_addr, aa->spa, 4);
	bcopy(&ia, aa->tpa, 4);

	cc = write(sd, buf, len);
	if (cc < 0) {
		perror("write(request)");
		exit(1);
	}
	if (cc != len) {
		fprintf(stderr,
			"truncated write(request) (%d != %d)\n",
			cc, len);
		exit(1);
	}
	if (dump) {
		printf("send request\n");
		dodump(aa, buf, len);
	}
}

int inatmarp(int sd, u_int16_t opcode, struct sockaddr_in *s_in, 
	     struct in_addr *ia, int dump)
{
  register int cc;
  u_char buf[1024];
  int request = 0, reply = 0;
  struct aar *aa = NULL;

  if (dump)
    setlinebuf(stdout);
  switch (opcode) {
  case AAREQUEST:		/* code = 8 */
    request = 1;
    break;
  case AAREPLY:			/* code = 9 */
    reply = 1;
    break;
  default:
    printf("%d : bad InATMARP opcode\n", opcode);
    return(-1);
  }
  if (sd <= 0) {
    printf("inatmarp: No available native ATM socket for ATM call\n");
    return(-2);
  }

  if (request) {
    dorequest(sd, aa, s_in, buf, dump);
  }
  
  cc = read(sd, (char *)buf, sizeof(buf));
  if (cc < 0) {
    perror("read");
    return(-1);
  }
  if (cc == 0) {
    fprintf(stderr, "closed ?\n");
    return(-1);
  }
  if (cc < sizeof(llcsnap) + sizeof(struct aar))
    return(1);
  if (bcmp(buf, llcsnap, sizeof(llcsnap)) != 0)
    return(2);
  aa = (struct aar *)(buf + sizeof(llcsnap));
  aa->hrd = ntohs(aa->hrd);
  aa->pro = ntohs(aa->pro);
  aa->op = ntohs(aa->op);
  if ((aa->hrd != 0x0013) ||
      (aa->pro != 0x0800) ||
      ((aa->op != AAREQUEST) && (aa->op != AAREPLY)) ||
      (aa->spln != 4) ||
      (aa->tpln != 4)) {
    if (dump)
      fprintf(stderr, "bad packet\n");
    return(3);
  }

  if (dump)
    dodump(aa, buf, cc);
  if (reply && (aa->op == AAREQUEST))
    doreply(sd, aa, s_in, ia, buf, dump, cc);
  return(0);
}
