/* localnsap.c ATM End System Address/Identifier handling */
 
/* Written November 1997 by Francis.Dupont@inria.fr, Mohsen.Souissi@inria.fr */
 

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

#include <sys/param.h>
#include <sys/socket.h>
#include <sys/file.h>
#include <sys/ioctl.h>
#include <sys/sysctl.h>

#include <net/if.h>
#include <net/if_dl.h>
#include <net/route.h>

#include <netinet/in.h>
#include <netnatm/natm.h>

#include <sys/time.h>
#include <sys/types.h>


#include "../linux-atm.h"
#include "atm.h"
#include "localnsap.h"


/* we assume for the moment there is only one sap per interface */

int mib[] = { CTL_NET, PF_ROUTE, 0, 0, NET_RT_IFLIST, 0 };
int size;
caddr_t buf;
static unsigned char atm_prefix[] = ATM_PREFIX;
struct if_msghdr *ifm;
struct sockaddr_dl *sdl;

static caddr_t ifinfo(char *name);
static caddr_t linksock(char *name);

caddr_t get_addr_esi(int itf) {
        caddr_t cp;
        caddr_t esip;
	char name[IFNAMSIZ+1];

	bzero(name, IFNAMSIZ+1);
	(void)snprintf(name, IFNAMSIZ, ENNAME, itf);

        if (sysctl(mib, 6, NULL, &size, NULL, 0) < 0) {
                perror("sysctl0");
                exit(1);
        }
        if ((buf = malloc(size)) == NULL) {
                fprintf(stderr, "malloc failed\n");
                exit(1);
        }
        if (sysctl(mib, 6, buf, &size, NULL, 0) < 0) {
                perror("sysctl");
                exit(1);
        }

        for (cp = buf; cp < buf + size; cp += ifm->ifm_msglen) {
                ifm = (struct if_msghdr *)cp;
                if ((ifm->ifm_type == RTM_IFINFO) && (esip = ifinfo(name)))
                        return(esip);
        }
        fprintf(stderr, "not found\n");
        return(NULL);
}

#define ROUNDUP(a) \
        ((a) > 0 ? (1 + (((a) - 1) | (sizeof(long) - 1))) : sizeof(long))
#define ADVANCE(x, n) (x += ROUNDUP((n)->sdl_len))

static
caddr_t ifinfo(char *name) {
        caddr_t cp;
        int len, i;

        cp = (caddr_t)(ifm + 1);
        len = ifm->ifm_msglen;

        for (i = 0; i < RTAX_MAX; i++)
                if (ifm->ifm_addrs & (1 << i)) {
                        sdl = (struct sockaddr_dl *)cp;
                        if ((i == RTAX_IFP) && (sdl->sdl_family == AF_LINK))
                                if ((cp = linksock(name)))
                                        return (cp);
                        ADVANCE(cp, sdl);
                }

        return (NULL);
}

static
caddr_t linksock(char *name) {
        caddr_t cp;

        if ((sdl->sdl_nlen == strlen(name)) &&
            (strcmp(sdl->sdl_data, name) == 0) &&
            (sdl->sdl_alen == 6)) {
                cp = (caddr_t)(sdl->sdl_data + sdl->sdl_nlen);
             return (cp);
        }
        return (NULL);
}


void get_local_addr(struct svc_addr *addr, int max, int itf)
{
	FILE *file;
	char fname[64];
	caddr_t esip;
	char line[MAX_ATM_NAME_LEN + 1];

	if (max <= 0)
		return;
	bzero(addr, sizeof(struct svc_addr) * max);
	(void) snprintf(fname, sizeof(fname), NSAPFILE, itf);
	file = fopen(fname, "r");
	if (!file && errno == ENOENT) {
		bcopy(atm_prefix, addr->prv, PREFIX_LEN);
		esip = get_addr_esi(itf);
		bcopy(esip, addr->prv + PREFIX_LEN, ESI_LEN);
		addr->prv[ATM_ESA_LEN - 1] = 0x00;   /* SEL zeroed */
		return;
	}
	/* Read the file */
	while (max) {
		if (!fgets(line, MAX_ATM_NAME_LEN, file))
			return;
		if (*line)
			line[strlen(line) - 1] = '\0';
		if (text2atm_svc(line, addr, sizeof(struct svc_addr),
				 T2A_SVC | T2A_LOCAL) >= 0) {
			addr++;
			max--;
		} else
			fprintf(stderr, "%s: bad nsap %s\n",
				fname, line);
	}
}
