/* Some ATM declarations from <linux/ *.h> for compatibility with linux */
 
/* Written 1995-1998 by Werner Almesberger, EPFL LRC/ICA */


#ifndef _LINUX_ATM_H
#define _LINUX_ATM_H

/* stuff from <linux/kernel.h> */
 
/*
 * from atm.h
 */

/* general ATM constants */
#define ATM_CELL_SIZE		    53	/* ATM cell size incl. header */
#define ATM_CELL_PAYLOAD	    48	/* ATM payload size */
#define ATM_MAX_VPI                 255  /* maximum VPI at the UNI */
#define ATM_MAX_VPI_NNI             4096  /* maximum VPI at the NNI */
#define ATM_MAX_VCI                 65535  /* maximum VCI */

/* "protocol" values for the socket system call */
#define ATM_NO_AAL	0		/* AAL not specified */
#define ATM_AAL0	13		/* "raw" ATM cells */
#define ATM_AAL1	1		/* AAL1 (CBR) */
#define ATM_AAL2	2		/* AAL2 (VBR) */
#define ATM_AAL34	3		/* AAL3/4 (data) */
#define ATM_AAL5	5		/* AAL5 (data) */
#define ATM_SAAL	12		/* signaling AAL */


/* Traffic description */

#define ATM_NONE	0		/* no traffic */
#define ATM_UBR		1
#define ATM_CBR		2
#define ATM_VBR		3
#define ATM_ABR		4
#define ATM_ANYCLASS	5		/* compatible with everything */

#define ATM_MAX_PCR	-1		/* maximum available PCR */

struct atm_trafprm {
	u_char		traffic_class;	/* traffic class (ATM_UBR, ...) */
	int		max_pcr;	/* maximum PCR in cells per second */
	int		pcr;		/* desired PCR in cells per second */
	int		min_pcr;	/* minimum PCR in cells per second */
	int		max_cdv;	/* maximum CDV in microseconds */
	int		max_sdu;	/* maximum SDU in bytes */
};

struct atm_qos {
	struct atm_trafprm txtp;	/* parameters in TX direction */
	struct atm_trafprm rxtp;	/* parameters in RX direction */
	u_char aal;
};

#define ESI_LEN         6

#define ATM_OC3_PCR	(155520000/270*260/8/53)
			/* OC3 link rate:  155520000 bps
			   SONET overhead: /270*260 (9 section, 1 path)
			   bits per cell:  /8/53
			   max cell rate:  353207.547 cells/sec */

/* MF: change_qos (Modify) flags */

#define ATM_MF_IMMED	1	/* Block until change is effective */
#define ATM_MF_INC_RSV	2	/* Change reservation on increase */
#define ATM_MF_INC_SHP	4	/* Change shaping on increase */
#define ATM_MF_DEC_RSV	8	/* Change reservation on decrease */
#define ATM_MF_DEC_SHP	16	/* Change shaping on decrease */
#define ATM_MF_BWD	32	/* Set the backward direction parameters */

#define ATM_MF_SET	(ATM_MF_INC_RSV | ATM_MF_INC_SHP | ATM_MF_DEC_RSV | \
			 ATM_MF_DEC_SHP | ATM_MF_BWD)

/* PVC addressing is in <netnatm/natm.h>, struct sockaddr_natm */

static __inline__ int atmpvc_addr_in_use(struct sockaddr_natm addr)
{
	return *addr.snatm_if || addr.snatm_vpi || addr.snatm_vci;
}

/* SVC addressing */

#define	ATM_ESA_LEN	20		/* ATM End System Address length */
#define ATM_E164_LEN	12		/* maximum E.164 number length */

#define ATM_AFI_DCC	0x39		/* DCC ATM Format */
#define ATM_AFI_ICD	0x47		/* ICD ATM Format */
#define ATM_AFI_E164	0x45		/* E.164 ATM Format */

struct svc_addr {
	/* unused addresses must be bzero'ed */
	u_char prv[ATM_ESA_LEN];	/* private ATM address */
	char pub[ATM_E164_LEN + 1];	/* public address (E.164) */
};

static __inline__ int atmsvc_addr_in_use(struct svc_addr addr)
{
	return *addr.prv || *addr.pub;
}


/*
 * SAP structures
 */

#define ATM_MAX_BLLI	3	/* maximum number of BLLI elements */
#define ATM_MAX_HLI	7	/* maximum high-layer information length */
#define ATM_HL_NONE	0	/* HL not specified */

struct atm_blli {
	u_char l2_proto;	/* layer 2 protocol */
	union {
	    struct {
		u_char mode;	/* mode of operation (ATM_IMD_xxx)*/
				/* 0 if absent */
		u_char window;	/* window size (k) */
				/* 1-127 (0 to omit) */
	    } itu;		/* ITU-T encoding */
	    u_char user;	/* user-specified l2 information */
	} l2;
	u_char l3_proto;	/* layer 3 protocol */
	union {
	    struct {
		u_char mode;	/* mode of operation (ATM_IMD_xxx) */
				/* 0 if absent */
		u_char def_size;/* default packet size (log2) */
				/* 4-12 (0 to omit) */
		u_char window;	/* packet window size */
				/* 1-127 (0 to omit) */
	    } itu;		/* ITU-T encoding */
	    u_char user;	/* user specified l3 information */
	    struct {		/* if l3_proto = ATM_L3_H310 */
		u_char term_type;	/* terminal type */
		u_char fw_mpx_cap;	/* forward multiplexing capability */
					/* only if term_type != ATM_TT_NONE */
		u_char bw_mpx_cap;	/* backward multiplexing capability */
					/* only if term_type != ATM_TT_NONE */
	    } h310;
	    struct {		/* if l3_proto = ATM_L3_TR9577 */
		u_char ipi;	/* initial protocol id */
		u_char snap[5];	/* IEEE 802.1 SNAP identifier */
				/* (only if ipi == NLPID_IEEE802_1_SNAP) */
	    } tr9577;
	} l3;
	struct atm_blli *next;	/* next BLLI or NULL */
				/* (undefined when used in atmsvc_msg) */
				/* ONLY USED IN OLD-STYLE API */
};


struct atm_bhli {
	u_char hl_type;		/* high layer information type */
	u_char hl_length;	/* length */
				/* (only if hl_type == ATM_HL_USER || */
				/* hl_type == ATM_HL_ISO) */
	u_char hl_info[ATM_MAX_HLI];	/* high layer information */
};


struct atm_sap {
	struct atm_bhli bhli;		/* local SAP, high-layer information */
	struct atm_blli blli[ATM_MAX_BLLI];	/* local SAP, low-layer info */
};

static __inline__ int blli_in_use(struct atm_blli blli)
{
	return blli.l2_proto || blli.l3_proto;
}

/* From atmsvc.h, added as_add and as_drop for p2m */

enum atmsvc_msg_type { as_catch_null, as_bind, as_connect, as_add, as_accept, 
		       as_reject, as_listen, as_okay, as_error, as_indicate,
		       as_drop, as_close, as_itf_notify, as_modify,
		       as_identify, as_terminate };

/* added to identify the ATM connection configuration */
typedef enum { p2p, root, leaf } CONN_CONF;

struct atmsvc_msg {
	enum atmsvc_msg_type type;
	u_long vcc;			/* to identify the SOCKET struct */
	u_long listen_vcc;		/* indicate */
	short ep_ref;			/* < 0 (p2p), */
					/* = 0 (first party of p2m), */
					/* > 0 for next parties (p2m) */
	int reply;			/* for okay and close:		*/
					/*   < 0: error before active	*/
					/*    (sigd has discarded ctx)	*/
					/*   ==0: success		*/
					/*   > 0: error when active	*/
					/*    (still need to close)	*/
	struct sockaddr_natm pvc;	/* indicate, okay (connect) */
	struct svc_addr local;		/* local SVC address */
	struct atm_qos qos;		/*  QOS parameters */
	struct atm_sap sap;		/* SAP */
	struct svc_addr svc;		/* SVC address */
};

#define SELECT_TOP_PCR(tp) ((tp).max_pcr && (tp).max_pcr != ATM_MAX_PCR ? \
  (tp).max_pcr : (tp).min_pcr ? (tp).min_pcr : ATM_MAX_PCR)

#endif
