/* passive.c - ATM signaling test passive client */

/* Written 11-1997 by Mohsen.Souissi@inria.fr, INRIA */

#include <sys/types.h>
#include <sys/socket.h>
#include <net/if.h>
#include <netinet/in.h>
#include <netnatm/natm.h>
#include <arpa/inet.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <signal.h>
#include <limits.h>
#include <sys/wait.h>

#include "../linux-atm.h"
#include "atm.h"
#include "atmd.h"

#include "inatmarp.h"
#include "ipatmlookup.h"
#include "callroute.h"
#include "io.h"

#define COMPONENT	"PASSIVE"
#define MAX_CON 1
#define BUF_LEN 40

int debug = 0, inatmarp_dump = 0;
int active;              /* = 1 if the client is active, 0 otherwise */

const char *sig_qos = NULL;
const char *dump_dir = NULL;
int psd[2];			/* socket pair descriptors */

static int init_listen(void)
{
    int  ret;
    enum atmsvc_msg_type type;

    ret = send_atmsigd(0, 0, -1, as_listen, 0, 
		       NULL, &local_addr[0], NULL, &qos_default_p2p);
    if (ret != 0) {
	printf("ERROR: init_listen: failed to send as_listen\n");
	return(ret);
    }
    if ((ret = poll_atmsigd(NULL, NULL, &type)) != 0) {
	printf("ERROR: init_listen: got trouble when receiving "
	       "from atmsigd\n");
	return(ret);
    }
    return(0);
}

static void menu_display(void) 
{
    printf("\n\n");
    printf("\t\t****************************************\n");
    printf("\t\t*          ATM SIGNALING API           *\n");
    printf("\t\t****************************************\n\n\n");
    printf("1) SHOW Current ATM Calls\n");
    printf("2) DELETE an IP Route\n");
    printf("3) CLOSE an ATM Call (deletes route if any)\n");
    printf("4) CLOSE ALL \n");
    printf("5) QUIT\n");
    printf("\nchoice: ");
}

static int user_input(void)
{
    int ret = 0, len, choice;
    char choice_str[BUF_LEN], vcc_str[BUF_LEN];
    u_char type, ret_code;
    u_long vcc;
    PIPE_MSG wmsg;

 menu:
    menu_display();
    if (fgets(choice_str, sizeof(choice_str), stdin) == NULL) {
	printf("I/O ERROR on MENU\n");
	/* goto menu; */
	abort();		/* something is going wrong */
    }
    if ((len = strlen(choice_str)) < 2) 
	goto menu;
    choice_str[len - 1] = '\0';	/* replacing '\n' */
    if ((ret = str2num(choice_str)) < 0) {
	printf("\nuser_input: str2num: %s : Not a number\n", choice_str);
	fflush(stdout);
	(void) sleep(2);
	goto menu;
    }
    choice = ret;
    type = (u_char) choice;
    switch (choice) {
    case 1:			/* show_all() */
	if ((ret = write(psd[1], &type, 1)) < 0) { /* send only the type */
	    fprintf(stderr, "user_input write: %s\n", strerror(errno));
	    goto menu;
	}
	if (ret != 1) {
	    printf("ERROR: user_input (case %d): bad write\n", type);
	    goto menu;
	}
	if ((ret = read(psd[1], &ret_code, 1)) != 1) {
	    printf("ERROR:  user_input: read error %s\n", strerror(errno));
	    goto menu;
	}
	if (ret_code != 0)
	    printf("user_input: got trouble when processing command (%d)\n", 
		   type);
	goto menu;
    case 2:			/* delete_ipatmroute(vcc, ...) */
    case 3:			/* active_close_atmcall(..., vcc, ...) */
	printf("\n vcc : ");
	if (fgets(vcc_str, sizeof(vcc_str), stdin) == NULL) 
	    goto menu;
	len = strlen(vcc_str);
	vcc_str[len - 1] = '\0'; /* replacing '\n' */
	if ( (len < 2) || ((ret = str2num(vcc_str)) < 0) ) {
	    printf("\nuser_input: str2num: %s : not a number", vcc_str);
	    fflush(stdout);
	    (void) sleep(2);
	    goto menu;
	}
	vcc = (u_long) ret;
	bzero(&wmsg, sizeof(wmsg));
	wmsg.type = type;
	wmsg.vcc = vcc;
	if ((ret = write(psd[1], &wmsg, sizeof(wmsg))) < 0) {
	    fprintf(stderr, "user_input write: %s\n", strerror(errno));
	    goto menu;
	}
	if (ret != sizeof(wmsg)) {
	    printf("ERROR: user_input : bad write\n");
	    goto menu;
	}
	if ((ret = read(psd[1], &ret_code, 1)) != 1) {
	    printf("ERROR:  user_input: read error %s\n", strerror(errno));
	    goto menu;
	}
	if (ret_code != 0)
	    printf("user_input: got trouble when processing command (%d)\n", 
		   type);
	goto menu;
    case 4:			/* close_all(...) */
	if ((ret = write(psd[1], &type, 1)) < 0) {
	    fprintf(stderr, "user_input write: %s\n", strerror(errno));
	    goto menu;
	}
	if (ret != 1) {
	    printf("ERROR: user_input (case %d): bad write\n", type);
	    goto menu;
	}
	if ((ret = read(psd[1], &ret_code, 1)) != 1) {
	    printf("ERROR:  user_input: read error %s\n", strerror(errno));
	    goto menu;
	}
	if (ret_code != 0)
	    printf("user_input: got trouble when processing command (%d)\n", 
		   type);
	goto menu;
    case 5:			/* QUIT */
	if ((ret = write(psd[1], &type, 1)) < 0) {
	    fprintf(stderr, "user_input write: %s\n", strerror(errno));
	    goto menu;
	}
	if (ret != 1) {
	    printf("ERROR: user_input (case %d): bad write\n", type);
	    goto menu;
	}
	if ((ret = read(psd[1], &ret_code, 1)) != 1) {
	    printf("ERROR:  input_setup: read error %s\n", strerror(errno));
	    goto menu;
	}
	switch(ret_code) {
	case 0:
	    return(1);		/* we are about to quit */
	case 1:
	    goto menu;
	default:
	    printf("user_input: got trouble when processing command (%d)\n", 
		   type);
	    goto menu;
	}
    default:
	printf("\nuser_input: %d BAD CHOICE\n", choice);
	fflush(stdout);
	(void) sleep(2);
	goto menu;
    }	
}

static int process_cmd(void)
{
    int ret = 0, len;
    ATMCALL *call;
    u_char type, ret_code;
    PIPE_MSG rmsg;
    pid_t pid;
    int status;

    if ((ret = read(psd[0], &type, 1)) != 1) {
	printf("ERROR:  process_cmd: read error %s\n", strerror(errno));
	ret_code = 0;
	if ((ret = write(psd[0], &ret_code, 1)) < 0) {
	    fprintf(stderr, "process_cmd write: %s\n", strerror(errno));
	    return(-1);
	} 
	return(0);
    }
  
    if ((type != 1) && (type != 4) && (type != 5)) {
	len = sizeof(rmsg);
	bzero(&rmsg, len);
	if ((ret = read(psd[0], (u_char *)&rmsg + 1, len - 1)) != len -1) {
	    printf("ERROR:  process_cmd: read error %s\n", strerror(errno));
	    ret_code = 0;
	    if ((ret = write(psd[0], &ret_code, 1)) < 0) {
		fprintf(stderr, "process_cmd write: %s\n", strerror(errno));
		return(-1);
	    } 
	    return(0);
	}
    }

    printf("\n\n");  
    switch (type) {
    case 1:
	show_atmcalls();
	ret_code = 0;
	break;
    case 2:
	ret = (call =  delete_ipatmroute(rmsg.vcc, NULL)) ? 0 : -1;
	ret_code = ret ? 1 : 0;
	break;
    case 3:
	ret = active_close_atmcall(0, rmsg.vcc, NULL);
	ret_code = ret ? 1 : 0;
	break;
    case 4:
	ret = close_all(0);
	ret_code = ret ? 1 : 0;
	break;
    case 5: 
	if (AtmCalls != NULL) {
	    printf("\nprocess_cmd: !!! STILL %d PENDING CALL(S)", NbCalls);
	    printf("\n==> GO BACK TO MENU AND CLOSE ALL CALLS\n");
	    (void) sleep(2);
	    ret_code = 1;
	    break;
	}
	ret_code = 0;
	break;
    default:
	printf("process_cmd: Unknown type of command: %d\n", type);
	ret_code = 1;
	fflush(stdout);
	(void) sleep(2);
    }
    if ((ret = write(psd[0], &ret_code, 1)) < 0) {
	fprintf(stderr, "process_cmd write: %s\n", strerror(errno));
	return(-1);
    }
    if ((type == 5) && (ret_code == 0)) {
	printf("\n!!! Terminating session\n");
	if ((pid = wait(&status)) < 0) {
	    fprintf(stderr, "wait error: %s\n", strerror(errno));
	    return(-1);
	}
	return(1);
    }
    return(0);
}

/* polling for messages from atmsigd, from peer clients and socket pair */

static int main_loop(void)               
{
    int fds, ret, fin = 0;
    pid_t pid;
    fd_set perm, set;
    ATMCALL *call;
    int sig_down = 0;
    enum atmsvc_msg_type type;

    if (signal(SIGPIPE, sig_pipe) == SIG_ERR)
	printf("sig_pipe: error\n");

    FD_ZERO(&perm);
    FD_SET(sig_sd, &perm);
    fds = FD_SETSIZE;
    gettimeofday(&now, NULL);

    if ((ret = socketpair(AF_UNIX, SOCK_STREAM, 0, psd)) < 0) {
	printf("_loop: failed to create a socketpair\n");
	return(-1);
    }
    if ((pid = fork()) < 0) {
	fprintf(stderr, "fork error: %s\n", strerror(errno));
	exit(1);
    }
    if (pid == 0) {		/* child */
	if ((fin = user_input()) == 1) {
	    printf("Child terminating session, exiting normally\n");
	    exit(0);
	}
	exit(1);
    }
    if (pid > 0) {		/* parent */
	while (fin <= 0) {
	    set = perm;
	    FD_SET(psd[0], &set);
	    for (call = AtmCalls; call; call =  call->next)
		if (call->sd > 0) 
		    FD_SET(call->sd, &set);
	    ret = select(fds, &set, NULL, NULL, next_timer());
	    if (ret < 0) {
		if (errno != EINTR) { 
		    perror("select");
		    return(-1);
		}
		continue;
	    }
	    printf("----------");
	    gettimeofday(&now, NULL);
      
	    if (FD_ISSET(sig_sd, &set)) { 
		if ((ret = recv_atmsigd(NULL, NULL, &type)) < 0) {
		    if (ret == -2) { 
			/* sigd has closed his tcp socket with client */
			sig_down = 1;
			printf("\n\nmain_loop: !!! CONNECTION WITH ATMSIGD "
			       "IS DOWN !!!\n");
			printf("TEARING DOWN CURRENT ATM CALLS\n");
			(void) close_all(sig_down);
			if (kill(pid, SIGTERM) < 0) /* kill the child  */
			    perror("kill");
		    }
		    return(-2);
		}
	    }
	    for (call = AtmCalls; call; call =  call->next) {
		if((call->sd > 0) && (FD_ISSET(call->sd, &set))) {
		    if ((ret =  recv_natm_peer(call, inatmarp_dump)) != 0) {
			printf("recv_natm_peer: failed to read message "
			       "from peer\n");
			/* return(-1); */	/* Don't return here */
		    }
		}
	    }

	    if (FD_ISSET(psd[0], &set))
		fin = process_cmd();
      
	    /* expire timers after handling messges to make sure we don't
	       time out unnecessarily because of scheduling delays */
	    expire_timers();
	}
	return(0);
    }
}


/* ------------------------------- main ...  ------------------------------- */

int main(void)
{
    int ret;

    active = 0;
    inatmarp_dump = 1;
    set_application("PASSIVE");
    dump_dir = NULL;
    set_verbosity(NULL, DIAG_DEBUG);
    diag(COMPONENT, DIAG_INFO, "\nATM signaling passive client "
#ifdef UNI30
	 "UNI 3.0"
#endif
#ifdef UNI31
	 "UNI 3.1"
#ifdef ALLOW_UNI30
	 "+3.0compat"
#endif
#endif
#ifdef UNI40
	 "UNI 4.0"
#ifdef Q2963_1
	 "+Q.2931.1"
#endif
#endif
	 "/AAL5, version " VERSION "\n");
  
    if ((ret = open_atmsigd()) != 0) {
	printf("ERROR: main: failed to open atmsigd\n");
	return(ret);
    }
    init_current_time();
    init_addr();
    CurrVcc = 0;
    if ((ret = init_listen()) != 0) {
	printf("ERROR: main: failed to init listen\n");
	return(ret);
    }
    ret = main_loop();
    close(sig_sd);
    return (ret);
}
