/* callroute.h - ip/atm calls and route management */
 
/* Written 1998 by Mohsen.Souissi@inria.fr, INRIA */
 
#ifndef CALLROUTE_H
#define CALLROUTE_H

#define MAX_CALLS 10       /* maximum number of ATM calls per client */
#define BUF_LEN 40

typedef enum { 
  null, connecting, accepting, connected, disconnecting
} ATMCALL_STATE;

typedef struct _call {
  u_long vcc;
  u_int8_t      vpi;
  u_int16_t     vci;
  CONN_CONF conf;          /* p2p, root or leaf */
  ATMCALL_STATE state;
  int sd;		   /* socket descriptor for native ATM connection */
  int route;	           /* this field has sense only in p2p case */
  char rname[NAME_LEN];    /* if p2p/leaf conf and route != 0, remote name */
  char ipaddr[IP6_LEN];    /* if p2p/leaf conf and route != 0, IPv6 address */

  struct _call *next;
  struct _call *prev;
  int nbactp;
  struct _party *parties;  /* used if root or leaf conf */
} ATMCALL;

typedef enum { ps_null, ps_add_init, ps_add_recv, ps_active,
			   ps_drop_init,  ps_drop_recv } PARTY_STATE;

/* this structure is quite similar to the one in sigd/proto.h */
typedef struct _party {
  u_short ep_ref;			/* endpoint reference must be >= 0 */
  PARTY_STATE pstate;
  char rname[NAME_LEN];

  /* --- some point-to-multipoint information ---------------------------- */
  struct _party *next;		/* next party */
  struct _party *prev;		/* previous party */
  struct _call *call;		/* back pointer to the ATMCALL struct */
} PARTY;

typedef struct {
  u_char type;
  u_long vcc;
  char name[NAME_LEN];
  u_short ep_ref;
  CONN_CONF conf;
} PIPE_MSG;

extern const char *AddrConf;
extern ATMCALL *AtmCalls;     /* pointer to the last established call */
extern int NbCalls;
extern u_long CurrVcc;
extern const char *CallStateName[];
extern const char *PartyStateName[];

PARTY *new_party(ATMCALL *call, u_short ep_ref, PARTY_STATE pstate, 
		 char *ipatm);
ATMCALL *new_atmcall(u_long vcc, u_int8_t vpi, u_int16_t vci, 
		     CONN_CONF conf, ATMCALL_STATE state, char *remote_name);
void append_atmcall(ATMCALL *call);
void show_atmcalls(void);
ATMCALL *atmcall_lookup(u_long vcc);
PARTY *party_lookup(u_long vcc, u_short ep_ref);

void free_party(PARTY *party);
void free_all_parties(ATMCALL *call);
void free_atmcall(ATMCALL *call);

int atmcall_setup(char *remote_ipatm, short ep_ref);
int active_add_party(ATMCALL *call, char *remote_name);

int add_ipatmroute(ATMCALL *call);
ATMCALL *delete_ipatmroute(u_long vcc, ATMCALL *callarg);
int new_natm_sock(ATMCALL *call);
int iponatm(int opcode, u_long vcc, int dump);

int active_drop_party(u_long vcc, u_short ep_ref, PARTY *partyarg);
int active_close_atmcall(int sig_down, u_long vcc, ATMCALL *callarg);
void passive_drop_party(u_long vcc, u_short ep_ref, PARTY *partyarg);
void passive_close_atmcall(u_long vcc, ATMCALL *callarg);
int close_all(int sig_down);

int sendrtmsg(int op, ATMCALL *callarg);

#endif

