/* fromsig.c - Processing of incoming messages from atmsigd */

/* Written 11-1997 by Mohsen.Souissi@inria.fr INRIA */

#include <sys/types.h>
#include <sys/socket.h>
#include <net/if.h>
#include <netinet/in.h>
#include <netnatm/natm.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <assert.h>

#include "../linux-atm.h"
#include "atm.h"
#include "atmd.h"
#include "ipatmlookup.h"
#include "callroute.h"
#include "io.h"


#define COMPONENT "FROMSIG"

int atm_cac(struct atmsvc_msg *msg)
{
    return(0); 
}

static int process_connect_sdu(struct atmsvc_msg *msg, ATMCALL *call)
{
    PARTY *party;

    if ((call == NULL) || (call->state != connecting)) {
	printf("WARNING: proces_connect_sdu: as_connect: unexpected "
	       "message\n");
	return(1);
    }
    call->vpi = msg->pvc.snatm_vpi;
    call->vci = msg->pvc.snatm_vci;
    call->state = connected;
    printf("%s pvc: [vpi, vci] = [%d, %d] ", 
	   as_name[msg->type], call->vpi, call->vci);
    if (call->conf == root) {
	party = call->parties;
	if ((party == NULL) ||
	    (msg->ep_ref != 0) ||
	    (party->pstate != ps_add_init)) {
	    printf("ERROR: Received as_connect (p2m) in bad conf or bad "
		   "party state\n");
	    (void) active_close_atmcall(0, call->vcc, call);
	    return(-1);
	}
	party->pstate = ps_active;
    }
    return(0);
}

static int process_indicate_sdu(struct atmsvc_msg *msg, ATMCALL *callarg)
{
    int ret;
    ATMCALL *newcall;
    CONN_CONF conf;
    short ep_ref;
    PARTY *newparty = NULL;

    ep_ref = msg->ep_ref;
    conf = (ep_ref >= 0) ? leaf : p2p;
    if ((ret = atm_cac(msg)) != 0) {
	/* CAC didn't accept the new call */
	printf("WARNING: process_indicate_sdu: call rejected\n");
	(void) send_atmsigd(msg->listen_vcc, msg->vcc, ep_ref, as_reject, 
			    0, NULL, NULL, NULL, NULL);
	return(1);
    }
    newcall = new_atmcall(msg->listen_vcc, msg->pvc.snatm_vpi, 
			  msg->pvc.snatm_vci, conf, accepting, NULL);
    if (newcall == NULL) {
	printf("ERROR: process_indicate_sdu: failed to allocate a new atmcall "
	       "structure\n");
	(void) send_atmsigd(msg->listen_vcc, msg->vcc, ep_ref, as_reject, 
			    0, NULL, NULL, NULL, NULL);
	return(-1);
    }
    if (conf == leaf) {
	printf("Incoming p2m setup %s: %s\n with pvc: [vpi, vci] = [%d, %d] "
	       ", <EPR = %d>\n", (ep_ref > 0) ? "(actually add_party)" : " ",  
	       as_name[msg->type], newcall->vpi, newcall->vci, ep_ref);
	newparty = new_party(newcall, (u_short) ep_ref, ps_add_recv, NULL);
	if (newparty == NULL) {
	    printf("ERROR: process_indicate_sdu: failed to allocate a new" 
		   "party structure\n");
	    (void) send_atmsigd(msg->listen_vcc, msg->vcc, ep_ref, as_reject, 
				0, NULL, NULL, NULL, NULL);
	    goto teardown;
	}
	newcall->nbactp++;
    } else
	printf("Incoming p2p setup %s pvc: [vpi, vci] = [%d, %d] \n", 
	       as_name[msg->type], newcall->vpi, newcall->vci);
    ret = send_atmsigd(msg->vcc, msg->listen_vcc, ep_ref, as_accept, 
		       0, NULL, NULL, NULL, NULL);
    if (ret != 0) {
	printf("ERROR: process_indicate_sdu: failed to send as_accept\n");
	(void) send_atmsigd(msg->listen_vcc, msg->vcc, ep_ref, as_reject, 
			    0, NULL, NULL, NULL, NULL);
	goto teardown;
    }
    append_atmcall(newcall);
    NbCalls++;
    return(0);
 teardown:
    if (newparty != NULL)
	free(newparty);
    free(newcall);
    return(-1);
}


static int process_accept_sdu(struct atmsvc_msg *msg, ATMCALL *callarg)
{
    ATMCALL *call;
    PARTY *party;
    int sd;
  
    if (callarg != NULL)
	printf("WARNING: process_accept_sdu: as_accept: unexpected message\n");
    if ((call = atmcall_lookup(msg->vcc)) == NULL) {
	printf("WARNING: process_accept_sdu: no matching ATM call for "
	       "vcc %ld\n", msg->vcc);
	return(1);
    }
    if (call->state != accepting) {
	printf("WARNING: process_accept_sdu: as_accept: unexpected message\n");
	return(1);
    }
    call->state = connected;
    if (call->conf == leaf) {
	party = call->parties;
	if ((party == NULL) || (party->pstate != ps_add_recv)) {
	    printf("ERROR: process_accept_sdu: (p2m) as_accept in bad conf or"
		   " bad party state\n");
	    goto teardown;
	}
	printf("a new ATM call leaf accepted with <vcc =%ld>\n", msg->vcc);
	party->pstate = ps_active;
    } else			/* p2p conf */
	printf("a new ATM call accepted with vcc %ld\n", msg->vcc);
    
    /* creating a NATM socket for the new ATM call */
    if ((sd = new_natm_sock(call)) < 0) {
	printf("ERROR: process_accept_sdu: failed to create a NATM socket\n");
	goto teardown;
    }
    call->sd = sd;
    return(0);

 teardown:
    (void) active_close_atmcall(0, call->vcc, call);
    return(-1);
}

static int process_add_sdu(struct atmsvc_msg *msg, PARTY *partyarg)
{
    PARTY *party;

    if ((party = partyarg) == NULL) {
	printf("WARNING: process_add_sdu: unexpected message\n");
	return(1);		/* ignore the message */
    }
    if ((party->call->state != connected) || (party->pstate != ps_add_init)) {
	printf("WARNING: Received as_add in bad call/party states <%s, %s>\n",
	       CallStateName[party->call->state], 
	       PartyStateName[party->pstate]);
	return(1);		/* ignore the message */
    }
    printf("process_add_sdu: add party succeeded [Vcc = %ld, EPR = %d]\n",
	   party->call->vcc, party->ep_ref);
    party->pstate = ps_active;

    return(0);
}

static int process_drop_sdu(struct atmsvc_msg *msg, PARTY *partyarg)
{
    PARTY *party;
    int ret = 0;

    if ((party = partyarg) != NULL) {
	if (party->pstate == ps_add_init) {
	    ret = -2;
	    goto passive_drop;
	}
	if  (party->pstate != ps_drop_init) 
	    printf("WARNING: process_drop_sdu: received as_drop in bad "
		   "party state %s \n", PartyStateName[party->pstate]);
	printf("dropping party [Vcc = %ld, EPR = %d] confirmed\n", 
	       party->call->vcc, party->ep_ref);
	return(0);          	/* dropping party follows function return */
    } else {			/* passive drop */
	if ((party = party_lookup(msg->vcc, msg->ep_ref)) == NULL) {
	    printf("WARNING: process_drop_sdu: no matching party for "
		   "[Vcc = %ld, EPR = %d]\n", msg->vcc, msg->ep_ref);
	    return(1);
	}
    }
 passive_drop:
    printf("passive dropping party [Vcc = %ld, EPR = %d]\n", 
	   party->call->vcc, party->ep_ref);
    passive_drop_party(party->call->vcc, party->ep_ref, party);
    return(ret);
}

static int process_close_sdu(struct atmsvc_msg *msg, ATMCALL *callarg)
{
    ATMCALL *call;
    int ret = 0;
  
    if ((call = callarg) != NULL) { /* active close confirm */
	if (call->state != disconnecting) {
	    ret = -2;
	    goto passive_close;
	}
	printf("closing atm call <Vcc = %ld> confirmed\n", call->vcc); 
	return(0);  /* closing call at client side follows function return */
    }
    /* else  passive close */
    if ((call = atmcall_lookup(msg->vcc)) == NULL) {
	printf("WARNING: process_close_sdu: PASSIVE CLOSE: No such call "
	       "<Vcc =%ld>, message ignored\n", msg->vcc);
	return(1);
    }

 passive_close:    
    if (call->state != connected)
	printf("WARNING: process_close_sdu: received CLOSE in bad "
	       "state: %s\n", CallStateName[call->state]);
    printf("process_close_sdu: PASSIVE CLOSE, <Vcc = %ld>, <reply = %d>\n", 
	   msg->vcc, msg->reply);
    passive_close_atmcall(call->vcc, call);
    return(0);
}


int dispatch(struct atmsvc_msg *msg, ATMCALL *callarg, PARTY *partyarg, 
	     enum atmsvc_msg_type *type)
{
    int ret;
  
    *type = msg->type;
    switch (msg->type) {
    case as_okay:
	if (msg->vcc != 0)	/* answer for "as_bind" or "as_listen" */ 
	    printf("WARNING: Received OK: unexpected message\n");
	return(0);
    case as_connect:		/* connect_conf sdu */
	ret = process_connect_sdu(msg, callarg);
	return(ret);
    case as_indicate:
	ret = process_indicate_sdu(msg, callarg);
	return(ret);
    case as_accept:		/* accept_conf sdu */
	ret = process_accept_sdu(msg, callarg);
	return(ret);
    case as_add:		/* add_conf sdu */
	ret = process_add_sdu(msg, partyarg);
	return(ret);
    case as_error: 
	printf("ERROR RECEIVED, Clearing end point\n");
	return(0);
	/* passive drop or drop_conf sdu if requested by client */
    case as_drop:
	ret = process_drop_sdu(msg, partyarg);
	return(ret);
	/* passive close or close_conf sdu if requested by client */
    case as_close:
	ret = process_close_sdu(msg, callarg);
	return(ret);
    default:
	printf("WARNING: invalid message %d, message ignored\n", 
	       (int) msg->type);
	return(1);
    }
}


int from_atmsigd(struct atmsvc_msg *msg, ATMCALL *callarg, PARTY *partyarg, 
		 enum atmsvc_msg_type *type)
{
    int ret;

    printf("FROM_SIG: %s <vcc %ld, listen_vcc %ld>,  <EPR = %d>\n", 
	   as_name[msg->type], msg->vcc, msg->listen_vcc, msg->ep_ref);
    ret = dispatch(msg, callarg, partyarg, type);
    return(ret);
}
