/* io.c - I/O operations */

/* Written 1997-1998 by Mohsen.Souissi@inria.fr, INRIA */


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>

#include <sys/param.h>
#include <sys/socket.h>
#include <sys/file.h>
#include <sys/ioctl.h>
#include <sys/sysctl.h>

#include <net/if.h>
#include <net/if_dl.h>
#include <net/route.h>

#include <netinet/in.h>
#include <arpa/inet.h>
#include <netnatm/natm.h>

#include <sys/time.h>
#include <sys/types.h>
#include <netdb.h>

#include "../linux-atm.h"
#include "atm.h"
#include "atmsap.h"
#include "atmd.h"
#include "localnsap.h"
#include "ndp.h"
#include "ipatmlookup.h"
#include "callroute.h"
#include "io.h"
/* #include "../qgen/uni.h" */

#define COMPONENT "IO"


#define SERV_IP "0.0.0.0"
#define ATMSIG_PORT 6666


struct timeval now;
int sig_sd;

/* for atm qos_default_p2[pm] */
struct atm_qos qos_default_p2p = {{ATM_UBR, 0, 0, 0, 0},
				  {ATM_UBR, 0, 0, 0, 0},
				  ATM_AAL5};

struct atm_qos qos_default_p2m = {{ATM_UBR, 0, 0, 0, 0},
				  {ATM_NONE, 0, 0, 0, 0},
				  ATM_AAL5};

/* ---------------------------- addresses ------------------------------*/
/* we assume for the moment there is only one sap per interface */

struct svc_addr local_addr[MAX_ADDR];

struct svc_addr *get_addr(u_char *aesa, u_char sel)
{
    struct svc_addr *addr;
  
    addr = alloc(sizeof(struct svc_addr));
    *addr->pub = 0;
    bcopy(aesa, addr->prv, PREFIX_LEN + ESI_LEN);
    addr->prv[ATM_ESA_LEN -1] = sel; /* SEL zeroed */
    return(addr);
}

void init_addr(void)
{
    int i, j;

    get_local_addr(&local_addr[0], MAX_ADDR, 0);

    for (i = 1; i < MAX_ADDR; i++) {
	bcopy(&local_addr[0].prv[0], &local_addr[i], PREFIX_LEN + ESI_LEN);
	local_addr[i].prv[PREFIX_LEN + ESI_LEN] = 0x00 + (u_char)i;
	printf("\nLOCAL_ADDR[%d] = ", i);
	for (j = 0; j < PREFIX_LEN + ESI_LEN + 1; j++)
	    printf("%02X", local_addr[i].prv[j]);
	printf("\n");
    }
}

/* ---------------------- signal handling ------------------------------ */

void sig_pipe(int signo)
{
    printf("SIGPIPE caught\n");
    exit(1);
}

/* ----------------------------- signaling messages -------------------- */

const char *as_name[] = { "<invalid>", "as_bind", "as_connect", "as_add",
			  "as_accept", "as_reject", "as_listen","as_okay", 
			  "as_error", "as_indicate", "as_drop", "as_close" };



/* -------------------- Interface with atmsigd ------------------------- */

/* open a TCP socket to communicate with the signaling daemon (atmsigd) */
int open_atmsigd(void)   
{
    int s;
    struct sockaddr_in server_in;

    if ((s = socket(PF_INET, SOCK_STREAM, 0)) < 0) {
	perror("socket");
	return (1);
    }
    memset(&server_in, 0, sizeof(server_in));
    server_in.sin_len = sizeof(server_in);
    server_in.sin_family = AF_INET;
    server_in.sin_addr.s_addr = inet_addr(SERV_IP);
    server_in.sin_port =  htons(ATMSIG_PORT);
    if (connect(s, (struct sockaddr *) &server_in, sizeof(server_in)) < 0) {
	perror("connect");
	return (1);
    }
    /* intializing sig_sd */
    sig_sd = s;
    return (0);
}


int recv_atmsigd(ATMCALL *callarg, PARTY *partyarg, 
		 enum atmsvc_msg_type *type)
{
    static long abuffer[sizeof(struct atmsvc_msg) / sizeof(long) + 1];
    u_char *buffer = (u_char *)abuffer;
    int size, ret;

    if ((size = read(sig_sd, buffer, sizeof(struct atmsvc_msg))) < 0) {
	printf("ERROR:  recv atmsigd: %s\n", strerror(errno));
	return(-1);	
    }
    if (size == 0) {
	printf("ERROR: recv atmsigd: tcp_socket (%d) closed by peer\n", 
	       sig_sd);
	return(-2);
    }
    if (size != sizeof(struct atmsvc_msg)) {
	printf("FATAL: atmsigd message has bad length (got %d < %d)\n",
	       size, sizeof(struct atmsvc_msg));
	return(-6);
    }
    ret = from_atmsigd((struct atmsvc_msg *) buffer, callarg, partyarg, type);
    if (ret < 0)
	return(-7);
    return(ret);
}

int send_atmsigd(u_long vcc, u_long listen_vcc, short ep_ref,  
		 enum atmsvc_msg_type type, int reply, 
		 const struct svc_addr *svc, const struct svc_addr *local, 
		 const struct atm_sap *sap, const struct atm_qos *qos)
{
    struct atmsvc_msg *msg;
    int ret;

    msg = alloc(sizeof(struct atmsvc_msg));
    memset(msg, 0, sizeof(*msg));
    msg->vcc = vcc;  
    msg->listen_vcc = listen_vcc;
    msg->ep_ref = ep_ref;
    msg->type = type;
    msg->reply = reply;
    memset(&msg->pvc, 0, sizeof(msg->pvc));
    if (qos != NULL) 
	msg->qos = *qos;
    if (local != NULL)
	msg->local = *local;
    if (svc != NULL) 
	msg->svc = *svc;
    if (sap == NULL)
	msg->sap.bhli.hl_type = ATM_HL_NONE;
    else
	msg->sap = *sap;
    ret = to_atmsigd(msg);
    free(msg);
    return(ret);
}

int to_atmsigd(struct atmsvc_msg *msg)
{
    int wrote;
   
    printf("TO ATMSIGD: %s, <reply %d>, <vcc %ld, listen_vcc %ld>\n",
	   as_name[msg->type], msg->reply, msg->vcc, msg->listen_vcc);
    if ((wrote = write(sig_sd, msg, sizeof(*msg))) == sizeof(*msg))
	return(0);
    if (wrote < 0) {
	fprintf(stderr, "to_atmsigd write: %s", strerror(errno));
	if (errno == EPIPE)
	    printf("ERROR:  atmsigd is down\n");
	return(wrote);
    }
    printf("ERROR: bad atmsigd write: wanted %d, wrote %d\n",
	   sizeof(*msg), wrote);
    return(wrote);
}

/* -------------------- Native ATM communication ------------------------- */

int recv_natm_peer(ATMCALL *call, int dump)
{
    int ret;

    if (call->route != 0) {
	char buf[1024];

	ret = read(call->sd, buf, sizeof(buf));
	if (ret < 0) {
	    fprintf(stderr, "read on ATM vcc %ld failed: %s\n",
		    call->vcc, strerror(errno));
	    return(ret);
	}
	if (ret == 0) {
	    printf("ATM vcc %ld closed ?\n", call->vcc);
	    return(ret);
	}
	printf("flush %d bytes from ATM vcc %ld\n", ret, call->vcc);
	return(0);
    }
    if ((ret = iponatm(NDP_PASSIVE, call->vcc, dump)) != 0) {
	printf("recv_natm_peer: iponatm failed\n");
	return(ret);
    }
    return(0);
}


/* ----- common part ------------------------------------------------------- */

void init_current_time(void)
{
    gettimeofday(&now, NULL);
}

/* polling messages from atmsigd */
int poll_atmsigd(ATMCALL *callarg, PARTY *partyarg, 
		 enum atmsvc_msg_type *type) 
{
    fd_set set;
    int fds, ret, sig_down = 0;
  
    FD_ZERO(&set);
    FD_SET(sig_sd, &set);
    fds = FD_SETSIZE;
    gettimeofday(&now, NULL);
    ret = select(fds, &set, NULL, NULL, next_timer());
    if (ret < 0) {
	if (errno != EINTR) 
	    perror("select");
	return(-1);
    }
    printf("\tWaiting for incoming messages.....\n");
    gettimeofday(&now, NULL);
    if (FD_ISSET(sig_sd, &set)) 
	if ((ret = recv_atmsigd(callarg, partyarg, type)) < 0) {
	    if (ret == -2) {
		/* sigd has closed its tcp socket with client */
		sig_down = 1;
		printf("\n\nmain_loop: !!! CONNECTION WITH ATMSIGD "
		       "IS DOWN !!!\n");
		printf("TEARING DOWN CURRENT ATM CALLS\n");
		(void) close_all(sig_down);close(sig_sd);
	    }
	    return(-2);
	}
    /* expire timers after handling messges to make sure we don't
       time out unnecessarily because of scheduling delays */
    expire_timers();
    return(ret);
}
