#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include "ipatmlookup.h"

int ip_version = 6;

int str2num(const char *str)
{
  int i, len, num;
  
  if (str == NULL)
    return(-1);
  len = strlen(str);
  num = 0;
  for (i = 0; i < len; i++) {
    if ((isdigit(str[i])) == 0) {
      return(-1);
    }
  else 
    num = 10*num + (str[i] - '0');
  }
  return(num);
}

int char2int(char c)
{
  if ((c >= '0') && (c <= '9'))
    return(c - '0');
  if ( (c >= 'a') && (c <= 'f'))
    return(c - 'a' + 10);
  if ( (c >= 'A') && (c <= 'F'))
    return(c - 'A' + 10);
  printf("Invalid argument %c\n", c);
  return(-1);
}

int string2byte(const char *str, u_char *dst)
{
  int i, j, first, second;
  
  if (dst == NULL) {
    printf("null destination\n");
    return(-1);
  }
  for (j = i = 0; i < strlen(str); i += 2, j++) {
    if (((first = char2int(str[i])) < 0) || 
	((second = char2int(str[i+1])) < 0)) {
      printf("Invalid argument: %s\n", str);
      return(-1);
    }
    else 
      dst[j] = (u_char) (16*first + second);
  }
  return(0);
}

int badip(char *addr)
{
  int num = 0, dots = 0, lenpart = 0;
  char *cp;

  for (cp = addr; (*cp != '\0'); cp++) {
    if (isdigit(*cp)) {
      num = 10*num + (*cp - '0');
      lenpart++;
    }
    else {
      if (*cp == '.') {
	dots++;
	if ((dots > 3) || (lenpart > 3) || (num > 255))
	  return(-1);
	else {
	  num = 0;
	  lenpart = 0;
	}
      }
      else
	return(-1);
    }
  }
  if ((dots != 3)|| (lenpart > 3) || (num > 255))
    return(-1);
  return(0);
}

static struct ipatment *getentry(FILE *file)
{
  struct ipatment *entry;
  int i, len;
  char c, strprefix[2*PREFIX_LEN], stresi[2*ESI_LEN];

  if ((feof(file) != 0) || (ferror(file) != 0))
    return(NULL);
  entry = (struct ipatment *) malloc(sizeof(struct ipatment));
  switch (ip_version) {
  case 4:
    for (i = 0; i < IP4_LEN; i++) {
      c = getc(file);
      if (isdigit(c) || (c == '.'))
	entry->ipaddr[i] = c;
      else {
	entry->ipaddr[i] = '\0';
	ungetc(c, file);
	break;
      }
    }
    break;
  case 6:
    for (i = 0; i < IP6_LEN; i++) {
      c = getc(file);
      if (isxdigit(c) || (c == ':'))
	entry->ipaddr[i] = c;
      else {
	entry->ipaddr[i] = '\0';
	ungetc(c, file);
	break;
      }
    }
    break;
  default:
    printf("getentry: %d : bad IP version\n", ip_version);
    return(NULL);
  }
  do {
    c = getc(file);
  } while(isblank(c));
  ungetc(c, file);
  
  fread(strprefix, sizeof(char), 2*PREFIX_LEN, file);
  strprefix[2*PREFIX_LEN] = '\0';
  if (string2byte(strprefix, entry->prefix) != 0) {
    printf("getentry: %s : bad format of prefix\n", strprefix);
    return(NULL);
  }
  do {
    c = getc(file);
  } while(isblank(c));
  ungetc(c, file);

  fread(stresi, sizeof(char), 2*ESI_LEN, file);
  stresi[2*ESI_LEN] = '\0';
  if (string2byte(stresi, entry->esi) != 0) {
    printf("getentry: %s : bad format of esi\n", stresi);
    return(NULL);
  }
  do {
    c = getc(file);
  } while(isblank(c));
  ungetc(c, file);

  fgets(entry->name, NAME_LEN, file);
  len = strlen(entry->name);
  entry->name[len -1] = '\0';
  return(entry);
}

u_char *getaesabyname(const char *ipatmpathconf, char *name)
{
  FILE *file;
  int cmp = 0;
  u_char *aesa;
  struct ipatment *entry;

  if ((file = fopen(ipatmpathconf,"r")) == NULL) {
    perror("fopen");
    return(NULL);
  }
  while ((feof(file) == 0) && (ferror(file) == 0)) {
    if ((entry = getentry(file)) == NULL) {
      fclose(file);
      return(NULL);
    }
    if ((cmp = strcmp(entry->name, name)) == 0) {
      aesa = (u_char *) malloc(AESA_LEN*sizeof(u_char));
      bcopy(entry->prefix, aesa, PREFIX_LEN);
      bcopy(entry->esi, aesa + PREFIX_LEN, ESI_LEN);
      aesa[AESA_LEN -1] = 0x00;
      fclose(file);
      return(aesa);
    }
  }
  fclose(file);
  return(NULL);
}             

char *getipbyname(const char *ipatmpathconf, char *name)
{
  FILE *file;
  int cmp = 0;
  char *ipaddr;
  struct ipatment *entry;

  if ((file = fopen(ipatmpathconf,"r")) == NULL) {
    perror("fopen");
    return(NULL);
  }
  while ((feof(file) == 0) && (ferror(file) == 0)) {
    if ((entry = getentry(file)) == NULL) {
      fclose(file);
      return(NULL);
    }
    if ((cmp = strcmp(entry->name, name)) == 0) {
      ipaddr = (char *) malloc(IP6_LEN*sizeof(char));
      strcpy(ipaddr, entry->ipaddr);
      fclose(file);
      return(ipaddr);
    }
  }
  fclose(file);
  return(NULL);
}             

char *getipbyesi(const char *ipatmpathconf, u_char *esip)
{
  FILE *file;
  int cmp = 0;
  char *ipaddr;
  struct ipatment *entry;

  if ((file = fopen(ipatmpathconf,"r")) == NULL) {
    perror("fopen");
    return(NULL);
  }
  while ((feof(file) == 0) && (ferror(file) == 0)) {
    if ((entry = getentry(file)) == NULL) {
      fclose(file);
      return(NULL);
    }
    if ((cmp = bcmp(entry->esi, esip, ESI_LEN)) == 0) {
      ipaddr = (char *) malloc(IP6_LEN*sizeof(char));
      strcpy(ipaddr, entry->ipaddr);
      fclose(file);
      return(ipaddr);
    }
  }
  fclose(file);
  return(NULL);
}             
